@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: SOAP::Transport::MAILTO.pm,v 0.39 2000/10/08 22:55:20 $
+# $Id: SOAP::Transport::MAILTO.pm,v 0.40 2000/10/15 18:20:55 $
 #
 # ======================================================================
 
@@ -12,7 +12,7 @@ package SOAP::Transport::MAILTO;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.39';
+$VERSION = '0.40';
 
 # ======================================================================
 
@@ -59,7 +59,7 @@ sub send_receive {
 
   my $msg = MIME::Lite->new(
     From       => $parameters{From},
-    To         => $uri->authority,
+    To         => $uri->to,
     'Reply-To' => $parameters{'Reply-To'} || $parameters{From},
     Subject    => $parameters{Subject},
     Type       => 'text/xml',
@@ -72,7 +72,7 @@ sub send_receive {
     
   MIME::Lite->send(map {exists $parameters{$_} ? ($_ => $parameters{$_}) : ()} 'smtp', 'sendmail');
   eval { local $SIG{__DIE__}; $msg->send };
-  my $code = $@;
+  (my $code = $@) =~ s/ at .*\n//;
 
   $self->code($code);
   $self->message($code);
